import { EntityEquipmentInventoryComponent, EquipmentSlot, system, world } from "@minecraft/server";
import { Armors } from "./armors/imports";

const cooldown = new Map();
const conditionCheck = new Map();

system.runInterval(() => {
    for (const player of world.getPlayers()) {
        if (!cooldown.has(player.id)) cooldown.set(player.id, new Map());
        if (!conditionCheck.has(player.id)) conditionCheck.set(player.id, new Map());
        const equipment = player.getComponent(EntityEquipmentInventoryComponent.componentId);

        Armors.map((armor) => {
            for (const slot of Object.keys(armor).slice(1)) {
                const item = equipment.getEquipment(slot);
                const { cooldown: iCooldown, command, effect, condition, once, lore } = armor[slot];
                
                item.setLore(lore.map(lore => "§r§9" + lore));
                equipment.setEquipment(slot, item);

                if (!item?.typeId.startsWith("pa") || !item?.typeId.substr(3).startsWith(armor.id)) continue;
                if (!condition(player) && abilityEnded(player, item) && !!iCooldown) {
                    cooldown.get(player.id)
                        .set(item?.typeId, system.currentTick + iCooldown * 20);
                    conditionCheck.get(player.id).delete(item?.typeId);
                }
                if (once && conditionCheck.get(player.id).has(item?.typeId)) continue;
                if (!condition(player) || isInCooldown(player, item?.typeId)) continue;

                cooldown.get(player.id).delete(item?.typeId);
                conditionCheck.get(player.id).set(item?.typeId, system.currentTick);
                // Run Piece command OR ignore
                command ? player.runCommandAsync(command[0]) : null;
                // Add Piece effect OR ignore
                effect ? player.addEffect(effect.id, effect.duration ?? 61, { amplifier: effect.amplifier ?? 0, showParticles: false }) : null;
            }
        });
    }
}, 20);

function isInCooldown(player, itemType) {
    return cooldown.get(player.id).get(itemType) > system.currentTick;
}

function abilityEnded(player, item) {
    return (
        !cooldown.get(player.id).has(item.typeId) && system.currentTick - conditionCheck.get(player.id).get(item.typeId) > 2
    );
}